;; Generated package description from standard-themes.el  -*- no-byte-compile: t -*-
(define-package "standard-themes" "1.0.2" "Like the default theme but more consistent" '((emacs "27.1")) :commit "da876fc9e22a3240a9cd4a544b38e56e36ac7664" :authors '(("Protesilaos Stavrou" . "info@protesilaos.com")) :maintainer '("Standard-Themes Development" . "~protesilaos/standard-themes@lists.sr.ht") :keywords '("faces" "theme" "accessibility") :url "https://git.sr.ht/~protesilaos/standard-themes")
